@@ -9,6 +9,10 @@ import ctypes
 from msmart.security import security
 from msmart.device import convert_device_id_int
 from msmart.device import device as midea_device
+try:
+    import xml.etree.cElementTree as ET
+except ImportError:
+    import xml.etree.ElementTree as ET
 
 if sys.version_info < (3, 5):
     print(
@@ -17,7 +21,7 @@ if sys.version_info < (3, 5):
     )
     sys.exit(1)
 
-VERSION = '0.1.22'
+VERSION = '0.1.23'
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -33,6 +37,16 @@ BROADCAST_MSG = bytearray([
     0x56, 0x9e, 0xb8, 0xec, 0x91, 0x8e, 0x92, 0xe5
 ])
 
+DEVICE_INFO_MSG = bytearray([
+    0x5a, 0x5a, 0x15, 0x00, 0x00, 0x38, 0x00, 0x04,
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x27, 0x33, 0x05,
+    0x13, 0x06, 0x14, 0x14, 0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x03, 0xe8, 0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+    0xca, 0x8d, 0x9b, 0xf9, 0xa0, 0x30, 0x1a, 0xe3,
+    0xb7, 0xe4, 0x2d, 0x53, 0x49, 0x47, 0x62, 0xbe
+])
+
 
 @click.command()
 @click.option("-d", "--debug", default=False, count=True)
@@ -60,16 +74,22 @@ def discover(debug: int):
     found_devices = {}
     _LOGGER.info("msmart version: {}".format(VERSION))
     _LOGGER.info(
-        "Sending Device Scan Broadcast, press CTRL-C to quit...")
+        "Sending Device Scan Broadcast...")
     for i in range(10):
         try:
             sock.sendto(BROADCAST_MSG, ("255.255.255.255", 6445))
+            sock.sendto(BROADCAST_MSG, ("255.255.255.255", 20086))
             while True:
                 data, addr = sock.recvfrom(512)
                 m_ip = addr[0]
-                m_id, m_type, m_sn, m_ssid, m_version = 'unknown', 'unknown', 'unknown', 'unknown', 'unknown'
-                if len(data) >= 104 and (data[:2].hex() == '5a5a' or data[8:10].hex() == '5a5a') and m_ip not in found_devices:
+                if m_ip in found_devices:
+                    continue
+                m_id, m_type, m_sn, m_ssid, m_version, m_support = 'unknown', 'unknown', 'unknown', 'unknown', 'unknown', 'unknown'
+                if len(data) >= 104 and (data[:2].hex() == '5a5a' or data[8:10].hex() == '5a5a'):
                     _LOGGER.info("Midea Local Data {} {}".format(m_ip, data.hex()))
+                    data_string = data.decode("utf-8", errors="replace")
+                    # _LOGGER.info(data.decode("utf-8", errors="replace"))
+                    # root=ET.fromstring(data.decode(encoding="utf-8", errors="replace"))
                     if data[:2].hex() == '5a5a':
                         m_version = 'V2'
                     if data[:2].hex() == '8370':
@@ -82,7 +102,7 @@ def discover(debug: int):
                     reply = _security.aes_decrypt(encrypt_data)
                     _LOGGER.info("Decrypt Reply: {} {}".format(m_ip, reply.hex()))
                     
-                    m_ip = str(reply[3]) + "." + str(reply[2]) + "." + str(reply[1]) + "." + str(reply[0])
+                    m_ip = '.'.join([str(i) for i in reply[3::-1]])
                     m_port = str(bytes2port(reply[4:8]))
                     m_sn = reply[8:40].decode("utf-8")
                     # ssid like midea_xx_xxxx net_xx_xxxx
@@ -91,21 +111,72 @@ def discover(debug: int):
 
                     m_type = m_ssid.split('_')[1]
                     
-                    m_support = support_test(m_ip, int(m_id))
+                    m_support = support_test(m_ip, int(m_id), int(m_port))
 
                     _LOGGER.info(
                         "*** Found a {} device - type: '0x{}' - version: {} - ip: {} - port: {} - id: {} - sn: {} - ssid: {}".format(m_support, m_type, m_version, m_ip, m_port, m_id, m_sn, m_ssid))
-                elif m_ip not in found_devices:
-                    _LOGGER.info("Maybe not midea local data {} {}".format(m_ip, data.hex()))
+
+                if data[:6].hex() == '3c3f786d6c20':
+                    m_version = 'V1'
+                    root=ET.fromstring(data.decode(encoding="utf-8", errors="replace"))
+                    child = root.find('body/device')
+                    m=child.attrib
+                    m_port, m_sn, m_type  = m['port'], m['apc_sn'], str(hex(int(m['apc_type'])))[2:]
+                    response = get_device_info(m_ip, int(m_port))
+                    m_id = get_id_from_response(response)
+
+                    _LOGGER.info(
+                    "*** Found a {} device - type: '0x{}' - version: {} - ip: {} - port: {} - id: {} - sn: {} - ssid: {}".format(m_support, m_type, m_version, m_ip, m_port, m_id, m_sn, m_ssid))
+  
 
         except socket.timeout:
             continue
         except KeyboardInterrupt:
             sys.exit(0)
 
+def get_id_from_response(response):
+    if response[64:-16][:6].hex() == '3c3f786d6c20':
+        xml = response[64:-16]
+        root=ET.fromstring(xml.decode(encoding="utf-8", errors="replace"))
+        child = root.find('smartDevice')
+        m=child.attrib
+        return int.from_bytes(bytearray.fromhex(m['devId']), 'little')
+    else:
+        return 0
 
-def support_test(device_ip, device_id: int):
-    _device = midea_device(device_ip, device_id)
+def get_device_info(device_ip, device_port: int):
+    # Create a TCP/IP socket
+    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    sock.settimeout(8)
+
+    try:
+        # Connect the Device
+        device_address = (device_ip, device_port)
+        sock.connect(device_address)
+
+        # Send data
+        _LOGGER.debug("Sending to {}:{} {}".format(
+            device_ip, device_port, DEVICE_INFO_MSG.hex()))
+        sock.sendall(message)
+
+        # Received data
+        response = sock.recv(512)
+    except socket.error:
+        _LOGGER.info("Couldn't connect with Device {}:{}".format(
+            device_ip, device_port))
+        return bytearray(0)
+    except socket.timeout:
+        _LOGGER.info("Connect the Device %s:%s TimeOut for 8s. don't care about a small amount of this. if many maybe not support".format(
+            device_ip, device_port))
+        return bytearray(0)
+    finally:
+        sock.close()
+    _LOGGER.debug("Received from {}:{} {}".format(
+        device_ip, device_port, message.hex()))
+    return response
+
+def support_test(device_ip, device_id: int, device_port: int):
+    _device = midea_device(device_ip, device_id, device_port)
     device = _device.setup()
     device.refresh()
     if device.support:
